@@ -1,12 +1,27 @@
-from typing import Any, Dict, NamedTuple
+from typing import Any, Mapping, NamedTuple, Optional
+
+from dagster._core.definitions.metadata.table import TableSchema
+
+
+class AirbyteTableMetadata:
+    def __init__(
+        self,
+        schema: TableSchema,
+        normalization_tables: Optional[Mapping[str, "AirbyteTableMetadata"]] = None,
+    ):
+        """
+        Contains metadata about an Airbyte table, including its schema and any created normalization tables.
+        """
+        self.schema = schema
+        self.normalization_tables = normalization_tables or dict()
 
 
 class AirbyteOutput(
     NamedTuple(
         "_AirbyteOutput",
         [
-            ("job_details", Dict[str, Any]),
-            ("connection_details", Dict[str, Any]),
+            ("job_details", Mapping[str, Any]),
+            ("connection_details", Mapping[str, Any]),
         ],
     )
 ):
